from __future__ import annotations

import dataclasses
import os
import platform
from typing import Callable, Union


PKG_PPN = "livetw"

STATIC_PATH = os.path.join(
    os.path.dirname(os.path.realpath(__file__)), "resources"
)


@dataclasses.dataclass
class Resource:
    name: str
    content: str


def load_resource(name: str) -> Resource:
    with open(os.path.join(STATIC_PATH, name), "r") as f:
        return Resource(name, f.read())


def set_default_env(name: str, value: str) -> None:
    if name not in os.environ:
        os.environ[name] = value


class Term:
    if platform.system() == "Windows":
        os.system("color")

    BLACK = "\033[30m"
    R = "\033[31m"
    G = "\033[32m"
    BG = "\033[1;32m"
    Y = "\033[33m"
    B = "\033[34m"
    M = "\033[35m"
    C = "\033[36m"
    W = "\033[37m"
    END = "\033[0;0m"
    NORM = "\033[1m"
    BOLD = "\033[1m"

    @staticmethod
    def info(*values: object, end: str = "\n", sep: str = " ") -> None:
        print(f"{Term.C}[info]{Term.END}", *values, end=end, sep=sep)

    @staticmethod
    def warn(*values: object, end: str = "\n", sep: str = " ") -> None:
        print(f"{Term.Y}[warn]{Term.END}", *values, end=end, sep=sep)

    @staticmethod
    def error(*values: object, end: str = "\n", sep: str = " ") -> None:
        print(f"{Term.R}[error]{Term.END}", *values, end=end, sep=sep)

    @staticmethod
    def blank(end: str = "\n") -> None:
        print(end=end)

    @staticmethod
    def confirm(message: str) -> bool:
        response = input(f"{message} [y/N] ").strip().lower()
        while True:
            if not response or response.startswith("n"):
                return False
            if response.startswith("y"):
                return True
            print(f"{Term.R}Invalid response{Term.END}")
            response = (
                input(f"{Term.C}{message}{Term.END} [y/N] ").strip().lower()
            )

    @staticmethod
    def ask(
        message: str, validator: Union[None, Callable[[str], bool]] = None
    ) -> str:
        while True:
            response = input(message).strip()
            if not validator or validator(response):
                return response

            Term.info(f"Invalid response: {response}")

    @staticmethod
    def ask_file(message: str, base_dir: Union[str, None] = None) -> str:
        file = input(message).strip()

        while True:
            if not file:
                print("Please enter a non-empty file path")
                file = input(message).strip()
                continue

            full_path = f"{base_dir}/{file}" if base_dir else file
            if os.path.isfile(full_path):
                return file

            Term.error(f"File not found: {full_path}")
            file = input(message).strip()

    @staticmethod
    def ask_dir(
        message: str,
        base_dir: Union[str, None] = None,
        default: Union[str, None] = None,
    ) -> str:
        dir = input(message).strip()

        while True:
            if not default and not dir:
                print("Please enter a non-empty dir path")
                dir = input(message).strip()
                continue

            if default and not dir:
                full_path = f"{base_dir}/{default}" if base_dir else default

                if os.path.isdir(full_path):
                    return default
                Term.error(f"'{full_path}' is not a dir")
                dir = input(message).strip()
                continue

            full_path = f"{base_dir}/{dir}" if base_dir else dir
            if os.path.isdir(full_path):
                return dir

            Term.error(f"'{full_path}' is not a dir")
            dir = input(message).strip()


PKG_PP = f"{Term.M}[{PKG_PPN}]{Term.END}"


def pkgprint(*values: object, end: str = "\n", sep: str = " ") -> None:
    print(f"{PKG_PP}", *values, end=end, sep=sep)



from __future__ import annotations

import dataclasses
import re
from typing import Any

import tomli

from flask_livetw.util import PKG_PP, Term

DEFAULT_FLASK_ROOT = "src"

DEFAULT_FOLDER_STATIC = "static"

DEFAULT_FOLDER_TEMPLATE = "templates"
DEFAULT_TEMPLATES_GLOB = "**/*.html"

DEFAULT_FILE_ROOT_LAYOUT = "layout.html"
DEFAULT_FILE_LIVE_RELOAD = ".dev/live_reload.js"
DEFAULT_FILE_GLOBALCSS = ".dev/global.css"
DEFAULT_FILE_TAILWIND = ".dev/tailwind.css"
DEFAULT_FILE_MINIFIED_TAILWIND = "tailwind_min.css"

DEFAULT_LIVERELOAD_HOST = "127.0.0.1"
DEFAULT_LIVERELOAD_PORT = 5678

DEFAULT_FLASK_HOST = None
DEFAULT_FLASK_PORT = None
DEFAULT_FLASK_EXCLUDE_PATTERNS = []


def get_pyproject_toml(base_dir: str | None = None) -> dict[str, Any] | None:
    path = (
        f"{base_dir.strip()}/pyproject.toml"
        if base_dir and base_dir.strip()
        else "pyproject.toml"
    )
    try:
        with open(path, "rb") as f:
            return tomli.load(f)
    except FileNotFoundError:
        Term.info(f"Could not find pyproject.toml at '{path}'")
        return None
    except tomli.TOMLDecodeError as e:
        Term.warn(f"Malformed pyproject.toml: {e}")
        return None


@dataclasses.dataclass
class Config:
    flask_root: str

    static_folder: str
    full_static_folder: str

    templates_folder: str
    full_templates_folder: str

    templates_glob: str
    full_templates_glob: str
    root_layout_file: str
    full_root_layout_file: str

    live_reload_file: str
    full_live_reload_file: str

    globalcss_file: str
    full_globalcss_file: str
    tailwind_file: str
    full_tailwind_file: str
    tailwind_minified_file: str
    full_tailwind_minified_file: str

    live_reload_host: str
    live_reload_port: int

    flask_host: str | None
    flask_port: int | None
    flask_exclude_patterns: list[str] | None

    @staticmethod
    def from_dict_with_defaults(
        src_dict: dict[str, Any], base_dir: str | None = None
    ) -> Config:
        flask_root = src_dict.get("flask_root", DEFAULT_FLASK_ROOT)
        static_folder = src_dict.get("static_folder", DEFAULT_FOLDER_STATIC)
        templates_folder = src_dict.get(
            "templates_folder", DEFAULT_FOLDER_TEMPLATE
        )

        if isinstance(base_dir, str) and base_dir != "":
            base_dir = base_dir.rstrip("/")
            full_static_folder = f"{base_dir}/{flask_root}/{static_folder}"
            full_templates_folder = (
                f"{base_dir}/{flask_root}/{templates_folder}"
            )
        else:
            full_static_folder = f"{flask_root}/{static_folder}"
            full_templates_folder = f"{flask_root}/{templates_folder}"

        templates_glob = src_dict.get("templates_glob", DEFAULT_TEMPLATES_GLOB)
        root_layout_file = src_dict.get(
            "root_layout_file", DEFAULT_FILE_ROOT_LAYOUT
        )
        live_reload_file = src_dict.get(
            "live_reload_file", DEFAULT_FILE_LIVE_RELOAD
        )
        globalcss_file = src_dict.get("globalcss_file", DEFAULT_FILE_GLOBALCSS)
        tailwind_file = src_dict.get("tailwind_file", DEFAULT_FILE_TAILWIND)
        tailwind_minified_file = src_dict.get(
            "tailwind_minified_file", DEFAULT_FILE_MINIFIED_TAILWIND
        )
        live_reload_host = src_dict.get(
            "live_reload_host", DEFAULT_LIVERELOAD_HOST
        )
        live_reload_port = src_dict.get(
            "live_reload_port", DEFAULT_LIVERELOAD_PORT
        )
        flask_host = src_dict.get("flask_host", DEFAULT_FLASK_HOST)
        flask_port = src_dict.get("flask_port", DEFAULT_FLASK_PORT)
        flask_exclude_patterns = src_dict.get(
            "flask_exclude_patterns", DEFAULT_FLASK_EXCLUDE_PATTERNS
        )

        config_dict: dict[str, Any] = {
            "flask_root": flask_root,
            "static_folder": static_folder,
            "full_static_folder": full_static_folder,
            "templates_folder": templates_folder,
            "full_templates_folder": full_templates_folder,
            "templates_glob": templates_glob,
            "full_templates_glob": f"{full_templates_folder}/{templates_glob}",
            "root_layout_file": root_layout_file,
            "full_root_layout_file": f"{full_templates_folder}/{root_layout_file}",
            "live_reload_file": live_reload_file,
            "full_live_reload_file": f"{full_static_folder}/{live_reload_file}",
            "globalcss_file": globalcss_file,
            "full_globalcss_file": f"{full_static_folder}/{globalcss_file}",
            "tailwind_file": tailwind_file,
            "full_tailwind_file": f"{full_static_folder}/{tailwind_file}",
            "tailwind_minified_file": tailwind_minified_file,
            "full_tailwind_minified_file": f"{full_static_folder}/{tailwind_minified_file}",
            "live_reload_host": live_reload_host,
            "live_reload_port": live_reload_port,
            "flask_host": flask_host,
            "flask_port": flask_port,
            "flask_exclude_patterns": flask_exclude_patterns,
        }

        return Config(**config_dict)

    @staticmethod
    def default(base_dir: str | None = None) -> Config:
        return Config.from_dict_with_defaults({}, base_dir)

    @staticmethod
    def try_from_pyproject_toml(base_dir: str | None = None) -> Config | None:
        if base_dir is None or base_dir == "":
            base_dir = "."
        else:
            base_dir = base_dir.rstrip("/")

        pyproject = get_pyproject_toml(base_dir)
        if pyproject is None:
            return None

        tool_config = pyproject.get("tool")
        if tool_config is None:
            return None

        flask_livetw_config = tool_config.get("flask-livetw")
        if flask_livetw_config is None:
            return None

        return Config.from_dict_with_defaults(flask_livetw_config, base_dir)

    @staticmethod
    def from_pyproject_toml(base_dir: str | None = None) -> Config:
        """Errors are ignored and default values are used instead"""
        if base_dir is None or base_dir == "":
            base_dir = "."

        pyproject = get_pyproject_toml(base_dir)
        if pyproject is None:
            pyproject = {}

        config = pyproject.get("tool", {})
        if not isinstance(
            config, dict
        ):  # pyright: ignore[reportUnnecessaryIsInstance]
            config: dict[str, Any] = {}

        config = config.get("flask-livetw", {})
        if not isinstance(
            config, dict
        ):  # pyright: ignore[reportUnnecessaryIsInstance]
            config = {}

        return Config.from_dict_with_defaults(config, base_dir)


def add_field(
    pyproject: dict[str, Any],
    key: str,
    new_config: Config,
    default: Any,
    acc: str,
) -> str:
    value = pyproject.get(key, getattr(new_config, key))
    if value == default:
        return acc

    value_type = type(value)
    if value_type == str:
        return f'{acc}\n{key} = "{value}"'

    return f"{acc}\n{key} = {value}"


def update_pyproject_toml(config: Config) -> int:
    try:
        with open("pyproject.toml", "rb") as f:
            pyproject = tomli.load(f)
    except FileNotFoundError:
        Term.info("Could not find pyproject.toml")
        pyproject = {}
    except tomli.TOMLDecodeError as e:
        Term.info(f"Malformed pyproject.toml: {e}")
        Term.info("Verify that the file is valid TOML")
        return 1

    ppconfig = pyproject.get("tool", {}).get("flask-livetw", {})

    new_config = """\n[tool.flask-livetw]"""
    new_config = add_field(
        ppconfig, "flask_root", config, DEFAULT_FLASK_ROOT, new_config
    )
    new_config = add_field(
        ppconfig, "static_folder", config, DEFAULT_FOLDER_STATIC, new_config
    )
    new_config = add_field(
        ppconfig,
        "templates_folder",
        config,
        DEFAULT_FOLDER_TEMPLATE,
        new_config,
    )
    new_config = add_field(
        ppconfig, "templates_glob", config, DEFAULT_TEMPLATES_GLOB, new_config
    )
    new_config = add_field(
        ppconfig,
        "root_layout_file",
        config,
        DEFAULT_FILE_ROOT_LAYOUT,
        new_config,
    )
    new_config = add_field(
        ppconfig,
        "live_reload_file",
        config,
        DEFAULT_FILE_LIVE_RELOAD,
        new_config,
    )
    new_config = add_field(
        ppconfig, "globalcss_file", config, DEFAULT_FILE_GLOBALCSS, new_config
    )
    new_config = add_field(
        ppconfig, "tailwind_file", config, DEFAULT_FILE_TAILWIND, new_config
    )
    new_config = add_field(
        ppconfig,
        "tailwind_minified_file",
        config,
        DEFAULT_FILE_MINIFIED_TAILWIND,
        new_config,
    )
    new_config = add_field(
        ppconfig,
        "live_reload_host",
        config,
        DEFAULT_LIVERELOAD_HOST,
        new_config,
    )
    new_config = add_field(
        ppconfig,
        "live_reload_port",
        config,
        DEFAULT_LIVERELOAD_PORT,
        new_config,
    )
    new_config = add_field(
        ppconfig, "flask_host", config, DEFAULT_FLASK_HOST, new_config
    )
    new_config = add_field(
        ppconfig, "flask_port", config, DEFAULT_FLASK_PORT, new_config
    )
    new_config = add_field(
        ppconfig,
        "flask_exclude_patterns",
        config,
        DEFAULT_FLASK_EXCLUDE_PATTERNS,
        new_config,
    )
    new_config += "\n"

    try:
        with open("pyproject.toml", "r") as f:
            pyproject_toml = f.read()
            if "[tool.flask-livetw]" in pyproject_toml:
                pyproject_toml = re.sub(
                    r"\[tool\.flask-livetw\][^[]*", new_config, pyproject_toml
                )
            else:
                pyproject_toml += new_config

        with open("pyproject.toml", "w") as f:
            f.write(pyproject_toml)
    except FileNotFoundError:
        Term.info("Could not find pyproject.toml")
        Term.info("Creating pyproject.toml...")
        with open("pyproject.toml", "w") as f:
            f.write(new_config)

    return 0


def ask_project_layout(app_source: str | None = None) -> Config:
    flask_root = app_source
    if flask_root is None or flask_root == "":
        flask_root = Term.ask_dir(
            f"{PKG_PP} Flask app root (relative to {Term.C}cwd/{Term.END}) [{DEFAULT_FLASK_ROOT}] ",
            default=DEFAULT_FLASK_ROOT,
        )

    static_folder = Term.ask_dir(
        f"{PKG_PP} Static folder (relative to {Term.C}cwd/{flask_root}/{Term.END}) [{DEFAULT_FOLDER_STATIC}] ",
        flask_root,
        DEFAULT_FOLDER_STATIC,
    )

    templates_folder = Term.ask_dir(
        f"{PKG_PP} Templates folder (relative to {Term.C}cwd/{flask_root}/{Term.END}) [{DEFAULT_FOLDER_TEMPLATE}] ",
        flask_root,
        DEFAULT_FOLDER_TEMPLATE,
    )

    templates_glob = (
        Term.ask(
            f"{PKG_PP} Templates glob (relative to {Term.C}cwd/{flask_root}/{templates_folder}/{Term.END}) [{DEFAULT_TEMPLATES_GLOB}] ",
        )
        or DEFAULT_TEMPLATES_GLOB
    )

    root_layout_file = (
        Term.ask(
            f"{PKG_PP} Root layout file (relative to {Term.C}cwd/{flask_root}/{templates_folder}/{Term.END}) [{DEFAULT_FILE_ROOT_LAYOUT}] ",
        )
        or DEFAULT_FILE_ROOT_LAYOUT
    )

    live_reload_file = (
        Term.ask(
            f"{PKG_PP} Live reload file (relative to {Term.C}cwd/{flask_root}/{static_folder}/{Term.END}) [{DEFAULT_FILE_LIVE_RELOAD}] ",
        )
        or DEFAULT_FILE_LIVE_RELOAD
    )

    globalcss_file = (
        Term.ask(
            f"{PKG_PP} Global css file (relative to {Term.C}cwd/{flask_root}/{static_folder}/{Term.END}) [{DEFAULT_FILE_GLOBALCSS}] ",
        )
        or DEFAULT_FILE_GLOBALCSS
    )

    tailwind_file = (
        Term.ask(
            f"{PKG_PP} TailwindCSS file (relative to {Term.C}cwd/{flask_root}/{static_folder}/{Term.END}) [{DEFAULT_FILE_TAILWIND}] ",
        )
        or DEFAULT_FILE_TAILWIND
    )

    tailwind_minified_file = (
        Term.ask(
            f"{PKG_PP} Minified TailwindCSS file (relative to {Term.C}cwd/{flask_root}/{static_folder}/{Term.END}) [{DEFAULT_FILE_MINIFIED_TAILWIND}] ",
        )
        or DEFAULT_FILE_MINIFIED_TAILWIND
    )

    if not flask_root or flask_root == ".":
        full_static_folder = static_folder
        full_templates_folder = templates_folder
    else:
        full_static_folder = f"{flask_root}/{static_folder}"
        full_templates_folder = f"{flask_root}/{templates_folder}"

    return Config(
        flask_root=flask_root,
        static_folder=static_folder,
        full_static_folder=full_static_folder,
        templates_folder=templates_folder,
        full_templates_folder=full_templates_folder,
        templates_glob=templates_glob,
        full_templates_glob=f"{full_templates_folder}/{templates_glob}",
        root_layout_file=root_layout_file,
        full_root_layout_file=f"{full_templates_folder}/{root_layout_file}",
        live_reload_file=live_reload_file,
        full_live_reload_file=f"{full_static_folder}/{live_reload_file}",
        globalcss_file=globalcss_file,
        full_globalcss_file=f"{full_static_folder}/{globalcss_file}",
        tailwind_file=tailwind_file,
        full_tailwind_file=f"{full_static_folder}/{tailwind_file}",
        tailwind_minified_file=tailwind_minified_file,
        full_tailwind_minified_file=f"{full_static_folder}/{tailwind_minified_file}",
        live_reload_host=DEFAULT_LIVERELOAD_HOST,
        live_reload_port=DEFAULT_LIVERELOAD_PORT,
        flask_host=DEFAULT_FLASK_HOST,
        flask_port=DEFAULT_FLASK_PORT,
        flask_exclude_patterns=DEFAULT_FLASK_EXCLUDE_PATTERNS,
    )


def main() -> int:
    config = Config.from_pyproject_toml()
    print(config)

    return 0


if __name__ == "__main__":
    raise SystemExit(main())



#!/usr/bin/env python
from __future__ import annotations

import argparse
import asyncio
import concurrent.futures
import datetime
import json
import os
import platform
import shlex
import subprocess
from typing import Set, Union

import dotenv
import websockets.legacy.protocol as ws_protocol
import websockets.server as ws_server


class Term:
    if platform.system() == "Windows":
        os.system("color")

    BLACK = "\033[30m"
    R = "\033[31m"
    G = "\033[32m"
    BG = "\033[1;32m"
    Y = "\033[33m"
    B = "\033[34m"
    M = "\033[35m"
    C = "\033[36m"
    W = "\033[37m"
    END = "\033[0;0m"
    NORMAL = "\033[1m"
    BOLD = "\033[1m"


def dev_print(*values: object):
    print(f"{Term.M}[dev]{Term.END}", *values)


def int_or_default(value: Union[str, None], default: int) -> int:
    if value is None:
        return default
    try:
        return int(value)
    except ValueError:
        return default


dotenv.load_dotenv()

LRWS_HOST = os.getenv("LIVE_RELOAD_WS_HOST", "127.0.0.1")
LRWS_PORT = int_or_default(os.getenv("LIVE_RELOAD_WS_PORT"), 5678)
TW_INPUT_PATH = os.getenv(
    "TW_INPUT_PATH",
    "{tailwind_input_placeholder}",
)
TW_OUTPUT_PATH = os.getenv(
    "TW_OUTPUT_PATH",
    "{tailwind_output_placeholder}",
)
TW_OUTPUT_PATH_BUILD = os.getenv(
    "TW_OUTPUT_PATH_BUILD",
    "{minified_tailwind_output_placeholder}",
)


LR_CONNECTIONS: Set[ws_server.WebSocketServerProtocol] = set()


async def handle_connection(websocket: ws_server.WebSocketServerProtocol):
    LR_CONNECTIONS.add(websocket)
    try:
        await websocket.wait_closed()
    finally:
        LR_CONNECTIONS.remove(websocket)


async def live_reload_server(host: str, port: int):
    async with ws_server.serve(handle_connection, host, port) as server:
        dev_print(
            f"Live reload {Term.G}ready{Term.END} on {Term.C}ws://{host}:{Term.BOLD}{port}{Term.END}"  # noqa: E501
        )

        await server.wait_closed()

        dev_print(f"Live reload {Term.G}closed{Term.END}")


def handle_tw_output(process: "subprocess.Popen[bytes]"):
    if process.stdout is None:
        return

    for line in iter(process.stdout.readline, b""):
        if process.poll() is not None:
            break

        if line.startswith(b"Done"):
            ws_protocol.broadcast(
                LR_CONNECTIONS,
                json.dumps(
                    {
                        "type": "TRIGGER_FULL_RELOAD",
                        "data": datetime.datetime.now().isoformat(),
                    }
                ),
            )

        print(f'{Term.C}[twcss]{Term.END} {line.decode("utf-8")}', end="")


def handle_flask_output(process: "subprocess.Popen[bytes]"):
    if process.stdout is None:
        return

    for line in iter(process.stdout.readline, b""):
        if process.poll() is not None:
            break

        print(f'{Term.G}[flask]{Term.END} {line.decode("utf-8")}', end="")


async def dev_server(cli: argparse.Namespace):
    def live_reload_coroutine():
        if cli.no_live_reload or cli.no_tailwind:
            return None

        host = (
            LRWS_HOST if cli.live_reload_host is None else cli.live_reload_host
        )
        port = (
            LRWS_PORT if cli.live_reload_port is None else cli.live_reload_port
        )

        return live_reload_server(host, port)

    def tw_cli_executor(
        loop: asyncio.AbstractEventLoop,
        pool: concurrent.futures.ThreadPoolExecutor,
    ):
        if cli.no_tailwind:
            return None

        input_arg = f"-i {TW_INPUT_PATH}"
        if cli.tailwind_input is not None:
            input_arg = f"-i {cli.tailwind_input}"

        output_arg = f"-o {TW_OUTPUT_PATH}"
        if cli.tailwind_output is not None:
            output_arg = f"-o {cli.tailwind_output}"

        minify_arg = "--minify" if cli.tailwind_minify else ""

        cmd = f"tailwindcss --watch {input_arg} {output_arg} {minify_arg}"

        process = subprocess.Popen(
            shlex.split(cmd), stdout=subprocess.PIPE, stderr=subprocess.STDOUT
        )

        return loop.run_in_executor(pool, handle_tw_output, process)

    def flask_server_executor(
        loop: asyncio.AbstractEventLoop,
        pool: concurrent.futures.ThreadPoolExecutor,
    ):
        if cli.no_flask:
            return None

        host_arg = (
            "--host" + cli.flask_host if cli.flask_host is not None else ""
        )

        port_arg = (
            "--port" + cli.flask_port if cli.flask_port is not None else ""
        )

        debug_arg = "--debug" if cli.flask_mode == "debug" else ""

        exclude_patterns = ["*/**/dev.py", "*/**/install_dev_mode.py"]
        if cli.flask_exclude_patterns is not None:
            exclude_patterns.extend(cli.flask_exclude_patterns)
        exclude_patterns_arg = (
            f'--exclude-patterns {";".join(exclude_patterns)}'
        )

        cmd = f"\
            flask run {host_arg} {port_arg} {debug_arg} {exclude_patterns_arg}"

        process = subprocess.Popen(
            shlex.split(cmd), stdout=subprocess.PIPE, stderr=subprocess.STDOUT
        )

        return loop.run_in_executor(pool, handle_flask_output, process)

    loop = asyncio.get_running_loop()

    with concurrent.futures.ThreadPoolExecutor(max_workers=3) as pool:
        maybe_future_like = (
            live_reload_coroutine(),
            tw_cli_executor(loop, pool),
            flask_server_executor(loop, pool),
        )

        futures = (
            future for future in maybe_future_like if future is not None
        )

        _ = await asyncio.gather(*futures, return_exceptions=True)


def minify_tailwindcss(cli: argparse.Namespace):
    input_arg = f"-i {TW_INPUT_PATH}"
    if cli.input is not None:
        input_arg = f"-i {cli.input}"

    output_arg = f"-o {TW_OUTPUT_PATH_BUILD}"
    if cli.output is not None:
        output_arg = f"-o {cli.output}"

    minify_arg = "--minify" if cli.minify else ""

    command = f"tailwindcss {input_arg} {output_arg} {minify_arg}"

    dev_print("Minifying tailwindcss for production...")

    build_result = subprocess.run(shlex.split(command))

    if build_result.returncode != 0:
        dev_print(f"Tailwind build for production {Term.R}fail{Term.END}")
        return build_result.returncode

    dev_print(f"Tailwind build for production {Term.G}ready{Term.END}")
    return build_result.returncode


def main():
    cli_args = cli().parse_args()

    if cli_args.command == "build":
        if "LIVETW_BUILD" not in os.environ:
            os.environ["LIVETW_BUILD"] = "TRUE"

        return minify_tailwindcss(cli_args)

    if cli_args.command == "dev":
        if "LIVETW_DEV" not in os.environ:
            os.environ["LIVETW_DEV"] = "TRUE"

        asyncio.run(dev_server(cli_args))
        return 0

    dev_print(f"Unknown command {Term.BOLD}{cli_args.command}{Term.END}")
    dev_print(f"Use {Term.BOLD}dev.py [-h | --help]{Term.END} for help.")
    return 1


def cli() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        description="Enhanced dev environment for flask apps.",
        allow_abbrev=True,
    )

    subparsers = parser.add_subparsers(
        title="commands",
        dest="command",
        help="availible commands",
        required=True,
    )

    parser_build = subparsers.add_parser(
        name="build",
        description="\
            Build the tailwindcss of the provided input as a single css file.",
        help="Build tailwindcss for production.",
        allow_abbrev=True,
        formatter_class=argparse.MetavarTypeHelpFormatter,
    )
    parser_build.add_argument(
        "-i",
        "--input",
        dest="input",
        type=str,
        help="Input path, accepts glob patterns.",
    )
    parser_build.add_argument(
        "-o", "--output", dest="output", type=str, help="Output path."
    )
    build_minify_group = parser_build.add_mutually_exclusive_group()
    build_minify_group.add_argument(
        "--minify", dest="minify", action="store_true", help="Minify output."
    )
    build_minify_group.add_argument(
        "--no-minify",
        dest="minify",
        action="store_false",
        help="Do not minify output.",
    )
    parser_build.set_defaults(minify=True)

    parser_dev = subparsers.add_parser(
        name="dev",
        description="""
            Extended dev mode for flask apps.
            By default runs the flask app in debug mode,
            tailwindcss in watch mode and live reload server.
        """,
        help="Run a development server.",
        allow_abbrev=True,
        formatter_class=argparse.MetavarTypeHelpFormatter,
    )
    parser_dev.add_argument(
        "--no-live-reload",
        dest="no_live_reload",
        action="store_true",
        default=False,
        help="Disable live reload server.",
    )
    parser_dev.add_argument(
        "-lrh",
        "--live-reload-host",
        dest="live_reload_host",
        type=str,
        help="Hostname for live reload server.",
    )
    parser_dev.add_argument(
        "-lrp",
        "--live-reload-port",
        dest="live_reload_port",
        type=int,
        help="Port for live reload server.",
    )

    parser_dev.add_argument(
        "--no-flask",
        dest="no_flask",
        action="store_true",
        default=False,
        help="Disable flask server.",
    )
    parser_dev.add_argument(
        "-fh",
        "--flask-host",
        dest="flask_host",
        type=str,
        help="Hostname for flask server.",
    )
    parser_dev.add_argument(
        "-fp",
        "--flask-port",
        dest="flask_port",
        type=int,
        help="Port for flask server.",
    )
    parser_dev.add_argument(
        "-fm",
        "--flask-mode",
        dest="flask_mode",
        choices=("debug", "no-debug"),
        default="debug",
        help="If debug mode is enabled, the flask server will be started with --debug flag. Default: debug.",  # noqa: E501
    )
    parser_dev.add_argument(
        "--flask-exclude-patterns",
        dest="flask_exclude_patterns",
        type=str,
        nargs="+",
        help="File exclude patterns for flask server. Base: */**/dev.py */**/install_dev_mode.py",  # noqa: E501
    )

    parser_dev.add_argument(
        "-nt",
        "--no-tailwind",
        dest="no_tailwind",
        action="store_true",
        default=False,
        help="Disable tailwindcss generation. If tailwindcss is disabled the live reload server will not be started.",  # noqa: E501
    )
    parser_dev.add_argument(
        "-ti",
        "--tailwind-input",
        dest="tailwind_input",
        type=str,
        help="Input path for global css file. Includes glob patterns.",
    )
    parser_dev.add_argument(
        "-to",
        "--tailwind-output",
        dest="tailwind_output",
        type=str,
        help="Output path for the generated css file.",
    )
    parser_dev.add_argument(
        "-tm",
        "--tailwind-minify",
        dest="tailwind_minify",
        action="store_true",
        default=False,
        help="Enables minification of the generated css file.",
    )

    return parser


if __name__ == "__main__":
    raise SystemExit(main())



from __future__ import annotations

import argparse
import os
import re
from typing import Sequence

from flask_livetw.config import (
    Config,
    ask_project_layout,
    update_pyproject_toml,
)
from flask_livetw.util import Term, load_resource, pkgprint

LIVE_RELOAD_SCRIPT = load_resource("live_reload.js")
TAILWIND_CONFIG = load_resource("tailwind.config.js")
GLOBAL_CSS = load_resource("global.css")
LAYOUT_TEMPLATE = load_resource("layout.html")


def generate_tailwind_config(content_glob: str) -> str:
    return TAILWIND_CONFIG.content.replace(
        "{content_glob_placeholder}", content_glob
    )


def add_content_glob(config: str, content_glob: str) -> str | None:
    content_glob_re = re.compile(r"content:\s*\[([^\]]*)\]")
    re_match = content_glob_re.search(config)
    if re_match is None:
        return None

    existing_globs = re_match.group(1)
    content_start, content_end = re_match.span(1)
    prev = config[:content_start]
    next = config[content_end:]
    if existing_globs.strip() == "":
        new_globs = f"'{content_glob}',"
    else:
        no_new_line = existing_globs.lstrip("\n")
        space = " " * (len(no_new_line) - len(no_new_line.lstrip()))
        existing_globs = no_new_line.rstrip(", \t\n")
        if space == "":
            new_globs = f"\n    {existing_globs},\n    '{content_glob}',\n  "
        else:
            new_globs = f"\n{existing_globs},\n{space}'{content_glob}',\n  "

    config = f"{prev}{new_globs}{next}"
    config = config.rstrip() + "\n"
    return config


def generate_live_reload_template(
    live_reload_file: str, tailwind_file: str, tailwind_min_file: str
) -> str:
    return (
        """
  {% if config.DEBUG %}
    <link rel="stylesheet" type="text/css" href="{{ url_for('static', filename=\'"""  # noqa: E501
        + tailwind_file
        + """\') }}">
    <script src="{{ url_for('static', filename=\'"""
        + live_reload_file
        + """\') }}" defer></script>
  {% else %}
    <link rel="stylesheet" type="text/css" href="{{ url_for('static', filename=\'"""  # noqa: E501
        + tailwind_min_file
        + """\') }}">
  {% endif %}
"""
    ).strip("\n")


def generate_layout_template(
    live_reload_file: str, tailwind_file: str, tailwind_min_file: str
) -> str:
    return LAYOUT_TEMPLATE.content.replace(
        "{live_reload_template_placeholder}",
        generate_live_reload_template(
            live_reload_file, tailwind_file, tailwind_min_file
        ),
    )


def configure_tailwind(content_glob: str) -> int:
    Term.blank()
    pkgprint("Configuring tailwindcss...")

    if os.path.exists(TAILWIND_CONFIG.name):
        Term.info("Detected existing configuration file")
        Term.info("Updating tailwindcss configuration file...")

        with open(TAILWIND_CONFIG.name, "r") as f:
            existing_config = f.read()

        config = add_content_glob(existing_config, content_glob)
        if config is None:
            Term.info("No content config found in existing tailwind.config.js")
            Term.info(f"Manually add '{content_glob}' to your content config.")
            return -1

        with open(TAILWIND_CONFIG.name, "w") as f:
            f.write(config)

        pkgprint("Tailwindcss configured")
        return 0

    config = generate_tailwind_config(content_glob)

    with open(TAILWIND_CONFIG.name, "w") as f:
        f.write(config)

    pkgprint("Tailwindcss configured")
    return 0


def generate_files(
    live_reload_file: str,
    globalcss_file: str,
) -> None:
    Term.blank()
    pkgprint("Generating files...")

    try:
        with open(live_reload_file, "w") as f:
            f.write(LIVE_RELOAD_SCRIPT.content)
    except FileNotFoundError:
        os.makedirs(os.path.dirname(live_reload_file), exist_ok=True)
        with open(live_reload_file, "w") as f:
            f.write(LIVE_RELOAD_SCRIPT.content)

    try:
        with open(globalcss_file, "w") as f:
            f.write(GLOBAL_CSS.content)
    except FileNotFoundError:
        os.makedirs(os.path.dirname(globalcss_file), exist_ok=True)
        with open(globalcss_file, "w") as f:
            f.write(GLOBAL_CSS.content)

    pkgprint("Files generated")


def update_layout(
    root_layout_file: str,
    live_reload_file: str,
    tailwind_file: str,
    tailwind_min_file: str,
) -> int:
    Term.blank()
    pkgprint("Updating layout...")

    try:
        with open(root_layout_file, "+r") as f:
            layout = f.read()
            if "</head>" not in layout:
                Term.error(
                    "Root layour is malformed, the </head> tag is missing. \
                        Please check your root layout file."
                )
                return 1

            layout = layout.replace(
                "</head>",
                generate_live_reload_template(
                    live_reload_file, tailwind_file, tailwind_min_file
                )
                + "\n</head>",
            )
            f.seek(0)
            f.write(layout)
            f.truncate()

            pkgprint("Root layout file updated")
            return 0
    except FileNotFoundError as e:
        Term.warn(e)
        os.makedirs(os.path.dirname(root_layout_file), exist_ok=True)
        with open(root_layout_file, "w") as f:
            f.write(
                generate_layout_template(
                    live_reload_file, tailwind_file, tailwind_min_file
                )
            )

    pkgprint("Root layout file created")
    return 0


def update_gitignore(static_folder: str, tailwind_file: str) -> None:
    Term.blank()
    pkgprint("Updating .gitignore...")

    content = f"""
# flask-livetw
{static_folder}/{tailwind_file}
"""
    try:
        with open(".gitignore", "a") as f:
            f.write(content)
    except FileNotFoundError:
        Term.info("Missing .gitignore file, creating one...")
        with open(".gitignore", "w") as f:
            f.write(content)

    pkgprint(".gitignore updated")


def initialize(config: Config, gitignore: bool) -> int:
    Term.blank()
    pkgprint("Initializing flask-livetw 🚀...")

    Term.blank()
    pkgprint("Updating pyproject.toml...")
    code = update_pyproject_toml(config)
    if code != 0:
        return code

    tailwind_code = configure_tailwind(config.full_templates_glob)
    if tailwind_code > 0:
        return tailwind_code

    generate_files(
        config.full_live_reload_file,
        config.full_globalcss_file,
    )

    code = update_layout(
        config.full_root_layout_file,
        config.live_reload_file,
        config.tailwind_file,
        config.tailwind_minified_file,
    )
    if code != 0:
        return code

    if gitignore:
        update_gitignore(config.full_static_folder, config.tailwind_file)

    Term.blank()

    if tailwind_code == 0:
        pkgprint("Initialization completed ✅")
        return 0

    pkgprint("Initialization almost completed")
    pkgprint(
        "Remember to add the content glob to your tailwind.config.js manually"
    )
    pkgprint(f"Glob: '{config.full_templates_glob}'")

    return 0


def init(cli: argparse.Namespace) -> int:
    # project_config = Config.from_pyproject_toml()

    if cli.default:
        init_config = Config.default()
    else:
        pkgprint("Describe your project layout:")
        init_config = ask_project_layout()

    return initialize(init_config, gitignore=cli.gitignore)


def add_command_args(parser: argparse.ArgumentParser) -> None:
    parser.add_argument(
        "-D",
        "--default",
        dest="default",
        action="store_true",
        default=False,
        help="use default values for all options",
    )

    parser.add_argument(
        "--gi",
        "--gitignore",
        dest="gitignore",
        action="store_true",
        default=False,
        help="update .gitignore to exclude dev related files",
    )


def add_command(
    subparser: argparse._SubParsersAction[argparse.ArgumentParser],
) -> None:
    parser = subparser.add_parser(
        name="init",
        description="""
        Initialize flask-livetw for the project.
        Adds the configuration to pyproject.toml and creates the necessary files.
        """,
        help="Initialize flask-livetw for the project.",
        allow_abbrev=True,
        formatter_class=argparse.MetavarTypeHelpFormatter,
    )

    add_command_args(parser)


def main(args: Sequence[str] | None = None) -> int:
    parser = argparse.ArgumentParser(
        description="Initialize flask-livetw in the current working directory.",
        allow_abbrev=True,
        formatter_class=argparse.MetavarTypeHelpFormatter,
    )

    add_command_args(parser)

    parsed_args = parser.parse_args(args)

    return init(parsed_args)


if __name__ == "__main__":
    raise SystemExit(main())



from __future__ import annotations

import argparse
from typing import Sequence

from flask_livetw import build_app, dev_server, initialize, local_install


def create_cli() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        prog="flask-livetw",
        description="CLI for flask-livetw commands.",
        allow_abbrev=True,
    )
    subparsers = parser.add_subparsers(
        title="commands",
        dest="command",
        required=True,
    )

    dev_server.add_command(subparsers)

    build_app.add_command(subparsers)

    initialize.add_command(subparsers)

    local_install.add_command(subparsers)

    return parser


def main(args: Sequence[str] | None = None) -> int:
    parsed_args = create_cli().parse_args(args)

    if parsed_args.command == "dev":
        return dev_server.dev(parsed_args)

    if parsed_args.command == "build":
        return build_app.build(parsed_args)

    if parsed_args.command == "init":
        return initialize.init(parsed_args)

    if parsed_args.command == "local-install":
        return local_install.local_install(parsed_args)

    return 0


if __name__ == "__main__":
    raise SystemExit(main())



from __future__ import annotations

import argparse
import os
import re
import shlex
import subprocess
from typing import Sequence

from flask_livetw.config import Config, ask_project_layout
from flask_livetw.util import PKG_PP, Term, load_resource, pkgprint

DEV_DEPENDENCIES = "pytailwindcss python-dotenv websockets"


LIVE_RELOAD_SCRIPT = load_resource("live_reload.js")
DEV_SCRIPT = load_resource("dev.py")
TAILWIND_CONFIG = load_resource("tailwind.config.js")
GLOBAL_CSS = load_resource("global.css")
LAYOUT_TEMPLATE = load_resource("layout.html")


def generate_tw_config(content_glob: str) -> str:
    return TAILWIND_CONFIG.content.replace(
        "{content_glob_placeholder}", content_glob
    )


def generate_dev_script(
    globalcss_file: str, twcss_file: str, minified_twcss_file: str
) -> str:
    return (
        DEV_SCRIPT.content.replace(
            "{tailwind_input_placeholder}", globalcss_file
        )
        .replace("{tailwind_output_placeholder}", twcss_file)
        .replace("{minified_tailwind_output_placeholder}", minified_twcss_file)
    )


def generate_live_reload_template(
    live_reload_file: str, twcss_file: str, minified_twcss_file: str
) -> str:
    return (
        """
  {% if config.DEBUG %}
    <link rel="stylesheet" type="text/css" href="{{ url_for('static', filename=\'"""  # noqa: E501
        + twcss_file
        + """\') }}">
    <script src="{{ url_for('static', filename=\'"""
        + live_reload_file
        + """\') }}" defer></script>
  {% else %}
    <link rel="stylesheet" type="text/css" href="{{ url_for('static', filename=\'"""  # noqa: E501
        + minified_twcss_file
        + """\') }}">
  {% endif %}
"""
    ).strip("\n")


def generate_layout_template(
    live_reload_file: str, twcss_file: str, minified_twcss_file: str
) -> str:
    return LAYOUT_TEMPLATE.content.replace(
        "{live_reload_template_placeholder}",
        generate_live_reload_template(
            live_reload_file, twcss_file, minified_twcss_file
        ),
    )


def install_dev_dependencies() -> int:
    Term.blank()
    pkgprint("Installing required dev dependencies...")

    poetry_cmd = shlex.split(f"poetry add --group=dev {DEV_DEPENDENCIES}")

    try:
        _ = subprocess.run(poetry_cmd, shell=True, check=True)
    except subprocess.CalledProcessError as e:
        Term.error(e)
        Term.info(
            "Dev dependencies installation failed, please install them manually"  # noqa: E501
        )
        return -1

    pkgprint("Dev dependencies installed")
    return 0


def configure_tailwind(content_glob: str) -> int:
    Term.blank()
    pkgprint("Configuring tailwindcss...")

    if os.path.exists(TAILWIND_CONFIG.name):
        Term.info("Detected existing configuration file")
        Term.info("Updating tailwindcss configuration file...")

        with open(TAILWIND_CONFIG.name, "r+t") as f:
            config = f.read()
            content_glob_re = re.compile(r"content:\s*\[([^\]]*)\]")
            re_match = content_glob_re.search(config)
            if re_match is None:
                Term.info(
                    "No content config found in existing tailwind.config.js"
                )
                Term.info("Add the following glob to the content config:")
                Term.info(f"'{content_glob}',")
                return -1

            existing_globs = re_match.group(1)
            content_start, content_end = re_match.span(1)
            prev = config[:content_start]
            next = config[content_end:]
            if existing_globs.strip() == "":
                new_globs = f"'{content_glob}',"
            else:
                no_new_line = existing_globs.lstrip("\n")
                space = " " * (len(no_new_line) - len(no_new_line.lstrip()))
                existing_globs = no_new_line.rstrip(", \t\n")
                if space == "":
                    new_globs = (
                        f"\n    {existing_globs},\n    '{content_glob}',\n  "
                    )
                else:
                    new_globs = (
                        f"\n{existing_globs},\n{space}'{content_glob}',\n  "
                    )

            config = f"{prev}{new_globs}{next}"
            config = config.rstrip() + "\n"
            f.seek(0)
            f.write(config)
            f.truncate()

        Term.info("Tailwindcss configuration file updated")
        pkgprint("Tailwindcss configured")
        return 0

    with open(TAILWIND_CONFIG.name, "w") as f:
        f.write(generate_tw_config(content_glob))

    Term.info("Tailwindcss configuration file created")
    pkgprint("Tailwindcss configuration complete")
    return 0


def generate_files(
    live_reload_file: str,
    globalcss_file: str,
    twcss_file: str,
    minified_twcss_file: str,
) -> None:
    with open(DEV_SCRIPT.name, "w") as f:
        f.write(
            generate_dev_script(
                globalcss_file, twcss_file, minified_twcss_file
            )
        )

    try:
        with open(live_reload_file, "w") as f:
            f.write(LIVE_RELOAD_SCRIPT.content)
    except FileNotFoundError:
        os.makedirs(os.path.dirname(live_reload_file), exist_ok=True)
        with open(live_reload_file, "w") as f:
            f.write(LIVE_RELOAD_SCRIPT.content)

    with open(globalcss_file, "w") as f:
        f.write(GLOBAL_CSS.content)


def update_layout(
    root_layout_file: str,
    live_reload_file: str,
    twcss_file: str,
    minified_twcss_file: str,
) -> int:
    try:
        with open(root_layout_file, "+r") as f:
            layout = f.read()
            if "</head>" not in layout:
                Term.error(
                    "Root layour is malformed, the </head> tag is missing. \
                        Please check your root layout file."
                )
                return 1

            layout = layout.replace(
                "</head>",
                generate_live_reload_template(
                    live_reload_file, twcss_file, minified_twcss_file
                )
                + "\n</head>",
            )
            f.seek(0)
            f.write(layout)
            f.truncate()
            return 0
    except FileNotFoundError as e:
        Term.warn(e)
        os.makedirs(os.path.dirname(root_layout_file), exist_ok=True)
        with open(root_layout_file, "w") as f:
            f.write(
                generate_layout_template(
                    live_reload_file, twcss_file, minified_twcss_file
                )
            )

    return 0


def update_gitignore(static_folder: str, twcss_file: str) -> None:
    content = f"""
# flask-livetw
{static_folder}/{twcss_file}
"""
    try:
        with open(".gitignore", "a") as f:
            f.write(content)
    except FileNotFoundError:
        Term.info("Missing .gitignore file, creating one...")
        with open(".gitignore", "w") as f:
            f.write(content)


def check_requirements() -> int:
    pkgprint("Checking requirements...")
    cwd = os.getcwd()
    pkgprint(f"Current working directory: {Term.C}{cwd}{Term.END}")
    continue_install = Term.confirm(f"{PKG_PP} Is this your project root?")

    if not continue_install:
        pkgprint("Change cwd and start again. Modding canceled")
        return 1

    python_cmd = shlex.split("python --version")
    python_cmd_result = subprocess.run(
        python_cmd, shell=True, capture_output=True
    )

    if python_cmd_result.returncode != 0:
        Term.error("python --version failed, is python installed?")
        return python_cmd_result.returncode

    version = python_cmd_result.stdout.decode("utf-8").strip()
    if version != "Python 3.8.10":
        pkgprint(f"python --version: {Term.C}{version}{Term.END}")

        continue_install = Term.confirm(
            f"{PKG_PP} Continue with this version?"
        )
        if not continue_install:
            pkgprint("Change python version and start again. Modding canceled")
            return 1

    return 0


def local_install(args: argparse.Namespace) -> int:
    if not args.all_yes:
        code = check_requirements()
        if code != 0:
            return code

    config = Config.try_from_pyproject_toml()
    if config is None:
        config = ask_project_layout()

    Term.blank()
    pkgprint("Installing flask-livetw as local script 🚀...")

    dependancies_code = install_dev_dependencies()
    if dependancies_code > 0:
        return dependancies_code

    tailwind_code = configure_tailwind(config.full_templates_glob)
    if tailwind_code > 0:
        return tailwind_code

    generate_files(
        config.full_live_reload_file,
        config.full_globalcss_file,
        config.full_tailwind_file,
        config.full_tailwind_minified_file,
    )

    code = update_layout(
        config.full_root_layout_file,
        config.live_reload_file,
        config.full_tailwind_file,
        config.full_tailwind_minified_file,
    )
    if code != 0:
        return code

    if args.gitignore:
        update_gitignore(config.full_static_folder, config.tailwind_file)

    Term.blank()

    if dependancies_code == 0 and tailwind_code == 0:
        pkgprint("Local install completed 🎉")
        return 0

    pkgprint("Local install almost completed")

    if dependancies_code != 0:
        pkgprint("Remember to install the missing dev dependencies manually")
        pkgprint(f"Dependancies: {DEV_DEPENDENCIES}")

    if tailwind_code != 0:
        pkgprint(
            "Remember to add the content glob to your tailwind.config.js manually"
        )
        pkgprint(f"Glob: '{config.full_templates_glob}'")

    return 0


def add_command_args(parser: argparse.ArgumentParser) -> None:
    parser.add_argument(
        "-Y",
        "--yes",
        dest="all_yes",
        action="store_true",
        default=False,
        help="answer yes to all requirements checks",
    )

    parser.add_argument(
        "-D",
        "--default",
        dest="default",
        action="store_true",
        default=False,
        help="use default values for all options",
    )

    parser.add_argument(
        "--gi",
        "--gitignore",
        dest="gitignore",
        action="store_true",
        default=False,
        help="update .gitignore to exclude dev related files",
    )


def add_command(
    subparser: argparse._SubParsersAction[argparse.ArgumentParser],
) -> None:
    parser = subparser.add_parser(
        name="local-install",
        description="""
        Install flask-livetw as a local script
        (adds dev dependencies, configures tailwindcss,
        adds dev script and updates root layout file).
        """,
        help="Install flask-livetw as a local script.",
        allow_abbrev=True,
        formatter_class=argparse.MetavarTypeHelpFormatter,
    )

    add_command_args(parser)


def main(args: Sequence[str] | None = None) -> int:
    parser = argparse.ArgumentParser(
        description="""
        Install flask-livetw as a local script
        (adds dev dependencies, configures tailwindcss,
        adds dev script and updates root layout file).
        """,
        allow_abbrev=True,
        formatter_class=argparse.MetavarTypeHelpFormatter,
    )
    add_command_args(parser)

    parsed_args = parser.parse_args(args)

    return local_install(parsed_args)


if __name__ == "__main__":
    raise SystemExit(main())



from __future__ import annotations

import argparse
import asyncio
import concurrent.futures
import dataclasses
import datetime
import json
import shlex
import subprocess
from typing import Sequence, Set

import websockets.legacy.protocol as ws_protocol
import websockets.server as ws_server

from flask_livetw.config import Config
from flask_livetw.util import Term, pkgprint, set_default_env


FLASK_BASE_EXCLUDE_PATTERNS = ("*/**/dev.py",)

LR_CONNECTIONS: Set[ws_server.WebSocketServerProtocol] = set()


async def handle_connection(websocket: ws_server.WebSocketServerProtocol):
    LR_CONNECTIONS.add(websocket)
    try:
        await websocket.wait_closed()
    finally:
        LR_CONNECTIONS.remove(websocket)


async def live_reload_server(host: str, port: int):
    async with ws_server.serve(handle_connection, host, port) as server:
        pkgprint(
            f"Live reload {Term.G}ready{Term.END} on {Term.C}ws://{host}:{Term.BOLD}{port}{Term.END}"
        )

        await server.wait_closed()

        pkgprint(f"Live reload {Term.G}closed{Term.END}")


def handle_tailwind_output(process: subprocess.Popen[bytes]):
    if process.stdout is None:
        return

    for line in iter(process.stdout.readline, b""):
        if process.poll() is not None:
            break

        if line.startswith(b"Done"):
            ws_protocol.broadcast(
                LR_CONNECTIONS,
                json.dumps(
                    {
                        "type": "TRIGGER_FULL_RELOAD",
                        "data": datetime.datetime.now().isoformat(),
                    }
                ),
            )

        print(f'{Term.C}[twcss]{Term.END} {line.decode("utf-8")}', end="")


def handle_flask_output(process: subprocess.Popen[bytes]):
    if process.stdout is None:
        return

    for line in iter(process.stdout.readline, b""):
        if process.poll() is not None:
            break

        print(f'{Term.G}[flask]{Term.END} {line.decode("utf-8")}', end="")


@dataclasses.dataclass
class DevConfig:
    no_live_reload: bool
    live_reload_host: str
    live_reload_port: int

    no_flask: bool
    flask_host: str | None
    flask_port: int | None
    flask_mode: str
    flask_exclude_patterns: Sequence[str] | None

    no_tailwind: bool
    tailwind_input: str | None
    tailwind_output: str
    tailwind_minify: bool


async def dev_server(config: DevConfig):
    def live_reload_coroutine():
        if config.no_live_reload or config.no_tailwind:
            return None

        host = config.live_reload_host
        port = config.live_reload_port

        return live_reload_server(host, port)

    def tailwind_cli_executor(
        loop: asyncio.AbstractEventLoop,
        pool: concurrent.futures.ThreadPoolExecutor,
    ):
        if config.no_tailwind:
            return None

        input_arg = ""
        if config.tailwind_input is not None:
            input_arg = f"-i {config.tailwind_input}"

        output_arg = f"-o {config.tailwind_output}"

        minify_arg = "--minify" if config.tailwind_minify else ""

        cmd = f"tailwindcss --watch {input_arg} {output_arg} {minify_arg}"

        process = subprocess.Popen(
            shlex.split(cmd), stdout=subprocess.PIPE, stderr=subprocess.STDOUT
        )

        return loop.run_in_executor(pool, handle_tailwind_output, process)

    def flask_server_executor(
        loop: asyncio.AbstractEventLoop,
        pool: concurrent.futures.ThreadPoolExecutor,
    ):
        if config.no_flask:
            return None

        host_arg = ""
        if config.flask_host is not None:
            host_arg = f"--host {config.flask_host}"

        port_arg = ""
        if config.flask_port is not None:
            port_arg = f"--port {config.flask_port}"

        debug_arg = "--debug" if config.flask_mode == "debug" else ""

        exclude_patterns: list[str] = list(FLASK_BASE_EXCLUDE_PATTERNS)
        if config.flask_exclude_patterns is not None:
            exclude_patterns.extend(config.flask_exclude_patterns)

        exclude_patterns_arg = (
            f"--exclude-patterns {';'.join(exclude_patterns)}"
        )

        cmd = f"\
            flask run {host_arg} {port_arg} {debug_arg} {exclude_patterns_arg}"

        process = subprocess.Popen(
            shlex.split(cmd), stdout=subprocess.PIPE, stderr=subprocess.STDOUT
        )

        return loop.run_in_executor(pool, handle_flask_output, process)

    loop = asyncio.get_running_loop()

    with concurrent.futures.ThreadPoolExecutor(max_workers=3) as pool:
        maybe_future_like = (
            live_reload_coroutine(),
            tailwind_cli_executor(loop, pool),
            flask_server_executor(loop, pool),
        )

        futures = (
            future for future in maybe_future_like if future is not None
        )

        pkgprint("Starting dev server...")

        _ = await asyncio.gather(*futures, return_exceptions=True)


def dev(cli_args: argparse.Namespace) -> int:
    set_default_env("LIVETW_DEV", "TRUE")

    project_config = Config.try_from_pyproject_toml()
    if project_config is None:
        pkgprint(
            "Project config not found. Dev server not started.",
        )
        pkgprint(
            "Try checking your current working directory or running 'flask-livetw init' to configure the project."
        )
        return 1

    no_live_reload = cli_args.no_live_reload
    live_reload_host = (
        cli_args.live_reload_host or project_config.live_reload_host
    )
    live_reload_port = (
        cli_args.live_reload_port or project_config.live_reload_port
    )

    no_flask = cli_args.no_flask
    flask_host = cli_args.flask_host or project_config.flask_host
    flask_port = cli_args.flask_port or project_config.flask_port
    flask_mode = cli_args.flask_mode
    flask_exclude_patterns = (
        cli_args.flask_exclude_patterns
        or project_config.flask_exclude_patterns
    )

    no_tailwind = cli_args.no_tailwind
    tailwind_input = (
        cli_args.tailwind_input or project_config.full_globalcss_file
    )
    tailwind_output = (
        cli_args.tailwind_output or project_config.full_tailwind_file
    )
    tailwind_minify = cli_args.tailwind_minify

    dev_config = DevConfig(
        no_live_reload=no_live_reload,
        live_reload_host=live_reload_host,
        live_reload_port=live_reload_port,
        no_flask=no_flask,
        flask_host=flask_host,
        flask_port=flask_port,
        flask_mode=flask_mode,
        flask_exclude_patterns=flask_exclude_patterns,
        no_tailwind=no_tailwind,
        tailwind_input=tailwind_input,
        tailwind_output=tailwind_output,
        tailwind_minify=tailwind_minify,
    )

    asyncio.run(dev_server(dev_config))
    return 0


def add_command_args(parser: argparse.ArgumentParser) -> None:
    parser.add_argument(
        "--no-live-reload",
        dest="no_live_reload",
        action="store_true",
        default=False,
        help="Disable live reload server.",
    )
    parser.add_argument(
        "-lrh",
        "--live-reload-host",
        dest="live_reload_host",
        type=str,
        help="Hostname for live reload server.",
    )
    parser.add_argument(
        "-lrp",
        "--live-reload-port",
        dest="live_reload_port",
        type=int,
        help="Port for live reload server.",
    )

    parser.add_argument(
        "--no-flask",
        dest="no_flask",
        action="store_true",
        default=False,
        help="Disable flask server.",
    )
    parser.add_argument(
        "-fh",
        "--flask-host",
        dest="flask_host",
        type=str,
        help="Hostname for flask server.",
    )
    parser.add_argument(
        "-fp",
        "--flask-port",
        dest="flask_port",
        type=int,
        help="Port for flask server.",
    )
    parser.add_argument(
        "-fm",
        "--flask-mode",
        dest="flask_mode",
        choices=("debug", "no-debug"),
        default="debug",
        help="If debug mode is enabled, the flask server will be started with --debug flag. Default: debug.",
    )
    parser.add_argument(
        "--flask-exclude-patterns",
        dest="flask_exclude_patterns",
        type=str,
        nargs="+",
        help="File exclude patterns for flask server. Base: */**/dev.py",
    )

    parser.add_argument(
        "--no-tailwind",
        dest="no_tailwind",
        action="store_true",
        default=False,
        help="Disable tailwindcss generation. If tailwindcss is disabled the live reload server will not be started.",
    )
    parser.add_argument(
        "-ti",
        "--tailwind-input",
        dest="tailwind_input",
        type=str,
        help="Input path for global css file. Includes glob patterns.",
    )
    parser.add_argument(
        "-to",
        "--tailwind-output",
        dest="tailwind_output",
        type=str,
        help="Output path for the generated css file.",
    )
    parser.add_argument(
        "-tm",
        "--tailwind-minify",
        dest="tailwind_minify",
        action="store_true",
        default=False,
        help="Enables minification of the generated css file.",
    )


def add_command(
    subparser: argparse._SubParsersAction[argparse.ArgumentParser],
) -> None:
    parser = subparser.add_parser(
        name="dev",
        description="""
        Extended dev mode for flask apps.
        By default runs the flask app in debug mode,
        tailwindcss in watch mode and live reload server.
        """,
        help="Run a development server.",
        allow_abbrev=True,
        formatter_class=argparse.MetavarTypeHelpFormatter,
    )

    add_command_args(parser)


def main(args: Sequence[str] | None = None) -> int:
    parser = argparse.ArgumentParser(
        description="""
        Extended dev mode for flask apps.
        By default runs the flask app in debug mode,
        tailwindcss in watch mode and live reload server.
        """,
        allow_abbrev=True,
        formatter_class=argparse.MetavarTypeHelpFormatter,
    )

    add_command_args(parser)

    parsed_args = parser.parse_args(args)

    return dev(parsed_args)


if __name__ == "__main__":
    raise SystemExit(main())



from __future__ import annotations

import argparse
import dataclasses
import shlex
import subprocess
from typing import Sequence

from flask_livetw.config import Config
from flask_livetw.util import Term, pkgprint, set_default_env


MINIFY_ON_BUILD = True


@dataclasses.dataclass
class BuildConfig:
    input: str
    output: str
    minify: bool


def minify_tailwind(config: BuildConfig) -> int:
    input_arg = f"-i {config.input}"

    output_arg = f"-o {config.output}"

    minify_arg = "--minify" if config.minify else ""

    command = f"tailwindcss {input_arg} {output_arg} {minify_arg}"

    pkgprint("Minifying tailwindcss for production...")

    build_result = subprocess.run(shlex.split(command))

    if build_result.returncode != 0:
        pkgprint(f"Tailwind build for production {Term.R}fail{Term.END}")
        return build_result.returncode

    pkgprint(f"Tailwind build for production {Term.G}ready{Term.END}")
    return build_result.returncode


def build(cli_args: argparse.Namespace) -> int:
    set_default_env("LIVETW_BUILD", "TRUE")

    config = Config.from_pyproject_toml()

    build_config = BuildConfig(
        input=cli_args.input or config.full_globalcss_file,
        output=cli_args.output or config.full_tailwind_minified_file,
        minify=cli_args.minify,
    )

    return minify_tailwind(build_config)


def add_command_args(parser: argparse.ArgumentParser) -> None:
    parser.add_argument(
        "-i",
        "--input",
        dest="input",
        type=str,
        help="Input path, accepts glob patterns.",
    )
    parser.add_argument(
        "-o", "--output", dest="output", type=str, help="Output path."
    )
    build_minify_group = parser.add_mutually_exclusive_group()
    build_minify_group.add_argument(
        "--minify", dest="minify", action="store_true", help="Minify output."
    )
    build_minify_group.add_argument(
        "--no-minify",
        dest="minify",
        action="store_false",
        help="Do not minify output.",
    )
    parser.set_defaults(minify=MINIFY_ON_BUILD)


def add_command(
    subparser: argparse._SubParsersAction[argparse.ArgumentParser],
) -> None:
    parser = subparser.add_parser(
        name="build",
        description="""
        Build the tailwindcss of the project as a single minified css file.
        """,
        help="Build tailwindcss for production.",
        allow_abbrev=True,
        formatter_class=argparse.MetavarTypeHelpFormatter,
    )

    add_command_args(parser)


def main(args: Sequence[str] | None = None) -> int:
    parser = argparse.ArgumentParser(
        description="""
        Build the tailwindcss of the project as a single css file.
        """,
        allow_abbrev=True,
        formatter_class=argparse.MetavarTypeHelpFormatter,
    )

    add_command_args(parser)

    parsed_args = parser.parse_args(args)

    return build(parsed_args)


if __name__ == "__main__":
    raise SystemExit(main())
